% Please cite the following paper if you are using this code or the
% accompanying RF-data 

% [1] Ashikuzzaman, M., Gauthier, C. J., Rivaz, H., "Global Ultrasound
% Elastography in Spatial and Temporal Domains", IEEE Trans. UFFC, 2019.

% Regularization coefficients:
% alpha1 = axial continuity in a A-line
% alpha2 = axial continuity at the neighboring samples
% alpha3 =  temporal continuity in the axial direction
% beta1 = lateral continuity in a A-line
% beta2 = lateral continuity at the neighboring samples
% beta3 =  temporal continuity in the lateral direction

% There are other parameters. The code is not very sensitive to these
% parameters, but if you want the best result, it is recommended to set
% them carefully:

% IRF: the range of variation of axial displacement. set it to:
% [-a 0] if compression
% [0 b] if extension
% [-a b] if not sure

% IA: the range of variation of lateral displacement. set it to:
% [-a 0] if moved to the left
% [0 b] if moved to the right
% [-a b] if not sure

% midA: the seed RF-line where the displacement calculation starts from

clear
% ------------------------------------------------------- %
% -------------- loading phantom RF Data ------------- %
% ------------------------------------------------------- %
load 'rf361'
Im2_new = BfData(301:end-250,:);
maxIm = max(Im2_new(:));
Im2_new = Im2_new/maxIm;

load 'rf360'
Im1_new = BfData(301:end-250,:);
Im1_new = Im1_new/maxIm;

load 'rf359'
Im0_new = BfData(301:end-250,:);
Im0_new = Im0_new/maxIm;


% AM2D parameters for initial estimates
IRF = [-10 10]; % Maximum allowed disparity in axial D
IA = [-4 4]; % Maximum allowed disparity in lateral D
midA = 50;
alfa = 5; %axial regularization
beta = 10; %lateral regularization
gamma = 0.005; %lateral regularization 
T = .2; % threshold for IRLS
xx = 1; % to compensate for attenuation
alfa_DP = 0.2; % DP regularization weight

% initial displacement estimates with AM2D
[ax0, lat0, ~] = RCF_AM2D(double(Im1_new), double(Im0_new), IRF, IA, midA, alfa_DP, alfa, beta, gamma, T, xx);
[ax1, lat1, ~] = RCF_AM2D(double(Im1_new),double(Im2_new), IRF, IA, midA, alfa_DP, alfa, beta, gamma, T, xx);

% % DP parameters for initial estimates
% IRF = [-10 10]; % Maximum allowed disparity in axial D
% IA = [-4 4]; % Maximum allowed disparity in lateral D
% alfa_DP = 5; % DP regularization weight
% 
% % initial displacement estimates with DP
% [ax0, lat0] = DP(double(Im1_new), double(Im0_new), IRF, IA, alfa_DP);
% [ax1, lat1] = DP(double(Im1_new), double(Im2_new), IRF, IA, alfa_DP);

a=zeros(1674,354,2);
l=zeros(1674,354,2);
Im=zeros(1674,354,3);
a(:,:,1) = -ax0(41:end-40, 16:end-15);
a(:,:,2) = ax1(41:end-40, 16:end-15);
l(:,:,1) = -lat0(41:end-40, 16:end-15);
l(:,:,2) = lat1(41:end-40, 16:end-15);
Im(:,:,1) = Im0_new(41:end-40, 16:end-15);
Im(:,:,2) = Im1_new(41:end-40, 16:end-15);
Im(:,:,3) = Im2_new(41:end-40, 16:end-15);


% regularization coefficients
alpha1= 5; 
alpha2= 1;
alpha3= 1.5;
beta1= 5; 
beta2= 1;
beta3= 1.5;


% refine the estimates with GUEST
tic
[Axial Lateral]=GUEST(a,l,Im,alpha1, alpha2,alpha3,beta1, beta2,beta3);
toc

% Axial strain
Axial2=Axial(:,:,2);
wDIff = 123; % window length of the differentiation kernel
strainA = LSQ(Axial2,wDIff);
strainA = -strainA((wDIff+1)/2:end-(wDIff-1)/2,:);
figure; imagesc([0 30],[0 30],strainA(:,61:end-30),[0.0001 0.0006]);
colorbar; colormap(gray);xlabel('width (mm)'); ylabel('depth (mm)');
ax = gca;
ax.FontSize = 16;